(function(window, App, $) {
    'use strict';
    //function adopted from HEX: hex.js
    var handleImageByAction = function(widget, action) {
        if (
            !widget.selectedObjects ||
            (widget.selectedObjects && widget.selectedObjects.length !== 1)
        ) {
            App.toaster.error('Please select single image');
            return false;
        }
        widget.loader.start();
        App.ajax
            .call({
                url: App.ajax.getUrl('imageManager', action),
                data: JSON.stringify([
                    widget.getFirstContextOrPageObjectId() || App.info.user.objectId,
                    widget.selectedObjects[0]
                ])
            })
            .then(function() {
                widget.refresh();
                App.page.refreshWidgetById('part');
                App.page.refreshWidgetById('ebom');
                App.page.refreshWidgetById('text');
            });
    };
    
    var submitUrlToHiddenFrame= function (url,objectId) {
        var f = document.getElementById('launchpadSubmitFrame');
        if (!f) {
            var e = document.createElement('DIV');
            e.id = 'launchpadSubmitActionDiv';
            e.style.position = 'absolute';
            e.style.top = '0px';
            e.style.left = '0px';
            e.style.height = '0px';
            e.style.width = '0px';
            e.style.visibility = 'hidden';
            f = document.createElement('IFRAME');
            f.id = 'launchpadSubmitFrame';
            f.name = 'launchpadSubmitFrame';
            f.src = parent.U_BLANK;
            f.height = '1px';
            f.width = '1px';
            f.border = '0';
            f.frameBorder = '0';
            e.appendChild(f);
            document.body.appendChild(f);
        }
        var finalURL;
        if (url.indexOf("?")>-1){
            finalURL=App.info.contextPath+url+"&objectId="+objectId;}
        else{
            finalURL=App.info.contextPath+url+"?objectId="+objectId;}
        f.contentWindow.document.location.href = finalURL;
    };
    
    var launchpad = {
         widget: {
            name: "",
            direction:"",
            relName:''
        },
    navigateToPage:function(data) {
       App.routing.entry(this.options.page); 
    },
        submitUrlFromCommandToHiddenFrame: function (data) {
            var selRow = this.selectedRows();
            var objectId = selRow[0].objectId;
            submitUrlToHiddenFrame (data.options.url,objectId);
        },
       
        submitUrlFromTableColumnToHiddenFrame: function (data) {
            submitUrlToHiddenFrame (data.url,data.row.objectId);
        },
        
        submitUrlFromCommandInFormToHiddenFrame: function (data) {
            var objectId = this.objectId;
            submitUrlToHiddenFrame (data.options.url,objectId);
        },


        /**
         * Use to load related widgets with current row id.
         * Refers an option named 'showWidgetIds' to get list of widgets to be shown
         * Refers an option named 'hideWidgetIds' to get list of widgets to be hidden
         * @param data
         */
        loadRelatedWidgets: function (data) {
            var payload = {};
            var updateConfig = {
              objectIds: [data.row.objectId]
            };
            
            if (App.Utils.isDefined(data.options.hideWidgetIds)) {
                var hideWidgetIds = data.options.hideWidgetIds.split(',');
                for (var i = 0; i < hideWidgetIds.length; i++) {
                    var hideWidgetId = hideWidgetIds[i];
                    var widgetToHide = this.widget.dashboard.getWidgetById(hideWidgetId);
                    try {
                        this.widget.dashboard.hideWidget(widgetToHide);
                    } catch (err) {}
                }
            }
            
            if (App.Utils.isDefined(data.options.showWidgetIds)) {
                var showWidgetIds = data.options.showWidgetIds.split(',');
                //store dashboard context oids (gets rewritten by method loadRelatedWidgets)
                var dashboardContextOIDs = this.widget.dashboard.context.objectIds;
                var pageObjectIds = App.page.context.objectIds; 
                App.page.context.objectIds = updateConfig.objectIds;
                
                for (var i = 0; i < showWidgetIds.length; i++) {
                    payload[showWidgetIds[i]] = updateConfig;
                }
                this.widget.loadRelatedWidgets(payload);
                
                //set back dahsboard context OIDs
                this.widget.dashboard.setContextObjectIds (dashboardContextOIDs);
                
                // set back page context object id.
                App.page.context.objectIds = pageObjectIds;
            }
        },
        /*FUNCTION adopted from pdr.js , for DOCUMENT revision -START*/
        revise: function(input) {
            var table = this;
            var obId = App.page.context.objectIds[0];

            App.ajax.call({
                url: App.ajax.getUrl('reviseDocService', 'revise'),
                dataType: 'json',
                type: 'post',
                async: false,
                data: JSON.stringify([obId, input.options.withfiles]) 
            }).then(function(data) {
                table.reloadContent();
                table.widget.refresh();
                App.toaster.info('New document revision was created.');
            }).catch(function (data) {
                var error = JSON.parse(data.responseText);
                App.toaster.error(error.error.message);
            });
        },
        /*FUNCTION adopted from pdr.js , for DOCUMENT revision -END*/        
        
        initRelatedObjectLinkForLoadRelatedWidgets: function() {
            var table = this;
            
            var _performAction = function (e) {
                 e.preventDefault();
                  var $link = $(e.currentTarget);
                  var onClick = $link.data('onclick');
                  var onClickParams = $link.data('onclick-params') || {};
                  var relatedObjectId = $link.data('related-object-id');
                  onClickParams.row = {"objectId":relatedObjectId};
                  onClickParams.event = e;
                  var fn = App.Utils.stringToFunction(onClick, window);

                  if (typeof(fn) === 'function') {
                    fn.call(this, onClickParams);
                  } else {
                    App.log.warning("Configured onClick handler (".concat(onClick, ") is not a function"));
                  }
            }
            table.dataTable.nodes().to$().find('div.he-column-link-load-related-widget > a').off('click').click(_performAction.bind(table));
        },
        
        /*FUNCTIONS adopted from HEX: hex.js - START */
        openPopup: function(url, width, height) {
            var defaultWidth = 800;
            var defaultHeight = 600;
            var w = width || defaultWidth;
            var h = height || defaultHeight;
            window.open(
                url,
                '',
                'width=' + w + ',height=' + h + ',resizable=yes,scrollbars=yes'
            );
        },

        loadPersonalTopPanel: function(input) {
            input.options.objectId = App.info.user.objectId;
            input.widget.$header
                .find('.title')
                .html(
                    input.widget.options.title +
                        ': ' +
                        App.info.user.firstName +
                        ' ' +
                        App.info.user.lastName
                );
            return App.form.create(input);
        },

        formHideEditButton: function() {
            this.element.find('.alpaca-form-button-toggleEditMode').hide();
        },

        setPrimaryImage: function() {
            handleImageByAction(this.widget, 'setPrimaryImage');
        },

        deleteImage: function() {
            handleImageByAction(this.widget, 'deleteImage');
        },

        showSlideshow: function() {
            App.launchpad.openPopup(
                App.info.actionURL +
                    'imageManagerSlideShow?objectId=' +
                    this.widget.getFirstContextOrPageObjectId(),
                1200,
                768
            );
        },

        loadImages: function() {
            $.each($('.tileProgressImage'), function(index, value) {
                var $image = $(value);
                var image = new Image();
                image.onload = function() {
                    $image.removeClass('tileProgressImage');
                    $image.removeAttr('style');
                    $image.attr('style', 'width: 100%; height: 100%');
                    $image.attr('src', this.src);
                    image = null;
                };
                image.onerror = function() {
                    $image.next().show();
                    $image.remove();
                    image = null;
                };
                image.src = $image.attr('data-src');
            });
        },

        openDashboardInSidePanel: function(data) {
            App.page.sidepanel.openDashboard(
                data.options.dashboardRef,
                data.row && data.row.objectId
            );
        },

        openDashboard2InSidePanel: function(data) {
            var selRow = this.selectedRows();
            var objectId = selRow[0].objectId;
            App.page.sidepanel.openDashboard(
                data.options.dashboardRef,
                objectId
            );
        },

        openDashboard3InSidePanel: function(objectId, dashboardRef) {
            App.page.sidepanel.openDashboard(dashboardRef, objectId);
        },

        openDashboardSidepanel : function(input) {
            var ref = input.options.dashboardRef;
            var oid = input.options.contextLess === true ? null : App.page.getObjectId();
            App.page.sidepanel.openDashboard(ref, oid);
        },

        onCodeDetect: function(result, context) {
            App.toaster.info('Scanned code: ' + result);
        },

        saveForOffline: function() {
            App.routing.openObjectsInBackground(this.getSelectedObjectIds());
        },

        toggleSidePanel : function() {
            App.page.sidepanel.toggle();
        },
        
        removePaginationForFewResults : function() {
            var THRESHOLD = 10;
            var numberOfRows = this.dataTable.data().length;
            if(numberOfRows <= THRESHOLD) {
                this.widget.$element.find(".he-dt-pagination").remove();
            }
        },
        
        addObject: function(input) {
            var projectRole = input.options.relationship;
            var searchConfig = input.options.searchConfig;
            var $widget = input.$widget;
            this.element.data('relationship', projectRole);
            this.element.data('table', this);
            if (!$widget.data('search-initilized')) {
                App.objectSearch = new App.Search(
                    {
                        searchContainer: $widget,
                        contextOid: this.widget ? this.widget.getFirstContextOrPageObjectId(): undefined
                    },
                    searchConfig,
                    this.element
                );
            }
            App.objectSearch.open();
        },
        
        addObjectSearchV2: function(config) {
            App.launchpad.widget.relName = config.options.relationship;
            App.launchpad.widget.direction= config.options.direction;
            App.launchpad.widget.name = this.widget.options.id;
        	App.searchV2.startSearch(config);
        },
        
        connectExistingObject: function (input) {
           var contextOid= App.page.context.objectIds[0];
           var searchHitOid =input.dataset.id;
           var rel=App.launchpad.widget.relName;
           var relDirection=App.launchpad.widget.direction;
           var widgetName = App.launchpad.widget.name;
           var args = JSON.stringify([contextOid, rel, relDirection, searchHitOid]);
            App.launchpad
                .connectObject(args)
                .then(function(message) {
                    App.toaster.success('Connected Object Successfully');
                    App.page.getWidgetById(widgetName).refresh();
            })["catch"] (
            function (e) {App.toaster.error(e.responseJSON.error.message); });
            this.close();
        },
        
        _doConnect: function (element, relDirection) {
            var rel = this.$element.data('relationship');
            var table = this.$element.data('table');
            var contextOid = this.options.contextOid || App.page.getObjectId();
            var contextObject = App.ObjectStore.get(searchHitOid);
            var searchHitOid = element.dataset.id;
            if (contextOid == searchHitOid) {
                App.toaster.error('Cannot connect two object with same TNR');
                return;
            }
            var args = JSON.stringify([contextOid, rel, relDirection, searchHitOid]);
            App.launchpad
                .connectObject(args)
                .then(function(message) {
                    table.reloadContent();
                    App.toaster.success('Connected Object Successfully');
                })
                ['catch'](function(e) {
                var msg = e.responseJSON.error.message;
                App.log.error(msg, {}, msg);
            });
        },

        connect: function(element) {
            App.launchpad._doConnect.call(this, element, 'to');
        },

        connectFrom: function (element) {
            App.launchpad._doConnect.call(this, element, 'from');
        },

        connectObject: function(args) {
            // Args format sample:
            // [objectId1, relationType, direction, objectId2]
            return App.ajax.call({
                url: App.ajax.getUrl('heliumConnect', 'connect'),
                dataType: 'json',
                type: 'post',
                async: true,
                data: args
            });
        },

        disconnectObject: function(args) {
            return App.ajax.call({
                //TODo:Implement disconnectMultiple in ConnectService Java
                //url : App.ajax.getUrl('heliumConnect', 'disconnectMultiple'),
                url: App.ajax.getUrl('heliumConnect', 'disconnect'),
                dataType: 'json',
                type: 'post',
                async: true,
                data: args
            });
        },

        removeObject: function(input) {
            var relType = input.options.relationship;
            var data = this.selectedRows();
            var projectRole = input.options.relationship;
            if (data.length === 0) {
                App.log.error('Table: No row selected');
                return;
            }
            var objectIds = [];
            for (var i = 0; i < data.length; i++) {
                objectIds.push(data[i].objectId);
            }
            var contextOid = App.page.getObjectId();
            if (this.widget) {
                contextOid = this.widget.getFirstContextOrPageObjectId();
            }
            var self = this;
            //var args = JSON.stringify([ contextOid,relType,'to',objectIds]);
            var args = JSON.stringify([
                contextOid,
                relType,
                'to',
                objectIds[0]
            ]);
            App.launchpad
                .disconnectObject(args)
                .then(function(msg) {
                    App.toaster.success('Disconnected Object Successfully');
                    self.reloadContent();
                })
                ['catch'](function(e) {
                    App.log.error('Remove failed. Error - ', {}, e);
                });
        },

        openObject: function openObject(input, search) {
            if (search) {
                search.close();
            }
            return App.routing.open(input.objectId, input.options.tab);
        },

        promoteObject: function promoteObject(input) {
            var oid = input.objectId || input.row.objectId || this.objectId || input;
            var dataTable =this;
            App.launchpad.promoteDemoteAjaxCall(oid, 'promote').then(function(msg) {
                if(msg.error && msg.error[0]) {
                    App.toaster.error('Object Promotion failed!. ' + msg.error[0]);
                } else {
                    App.toaster.info('Object Promoted!');
                    dataTable.reloadContent();
                }
            })['catch'](function (error) {
                return App.toaster.error('Object Promotion failed!. ' + error);
            });
        },

        demoteObject: function demoteObject(input) {
            var oid = input.objectId || input.row.objectId || this.objectId || input;
            var dataTable =this;
            App.launchpad.promoteDemoteAjaxCall(oid, 'demote').then(function(msg) {
                if(msg.error && msg.error[0]) {
                    App.toaster.error('Object Demotion failed!. ' + msg.error[0]);
                } else {
                    App.toaster.info('Object Demoted!');
                    dataTable.reloadContent();
                }
            })['catch'](function (error) {
                return App.toaster.error('Object Demotion failed!. ' + error);
            });
        },

        promoteDemoteAjaxCall: function promoteDemoteAjaxCall(objectIds, action) {
            var url = App.info.actionURL + 'heliumPromoteDemote';
            var params = $.extend(true, {}, { objectId: [objectIds], action: action }, {});
            return App.ajax.call({
                url: url,
                type: 'GET',
                data: params,
                traditional: true
            }, false);
        },
        
        clearCustomizations: function() {
            App.customization.customizationService.deleteAll()
            .then(function() {
                App.toaster.info("Cleared all customizations for " + App.info.user.id);
            });
        },
        
        notImplementedYet: function() {
            App.toaster.info("This functionality is not yet implemented");
        },
        
        startNSX: function(config) {
            var input = {
                    "options" : {
                        "config": config
                    }
            };
            App.searchV2.startSearch(input);
        },
        
        generateMobileIcons: function() {
            var arr = [
                {
                    "label": "Mes informations",
                    "icon": "ti-c ti-large ti-list-style-bullets",
                    "url": "javascript:App.routing.entry('tvc:page:helium:launchpad:common/Profile.xml')"
                },
                {
                    "label": "Recherche",
                    "icon": "ti-c ti-large ti-magnifying-glass",
                    "url": "javascript:App.launchpad.startNSX('tvc:search:helium:launchpad:common/TopBarSearchV2.xml');"
                },
                {
                    "label": "Mes projets favoris",
                    "icon": "ti-c ti-large ti-star",
                    "url": "javascript:App.routing.entry('tvc:page:helium:launchpad:project/MyProjects.xml');"
                },
                {
                    "label": "Mes projets",
                    "icon": "ti-c ti-large ti-worker",
                    "url": "javascript:App.routing.entry('tvc:page:helium:launchpad:project/MyProjects.xml');"
                },
                {
                    "label": "Mes tâches",
                    "icon": "ti-c ti-large ti-clipboard-checks",
                    "url": "javascript:App.routing.entry('tvc:page:helium:launchpad:project/MyTasks.xml')"
                },
                {
                    "label": "Travailler hors ligne",
                    "icon": "ti-c ti-large ti-signal-strength-0",
                    "url": "javascript:App.launchpad.notImplementedYet();"
                }
            ];
            var payload = [{
                "data": {
                    "image": {
                        "type": 0,
                        "editable": false,
                        "values": arr
                    },
                    "oid": "1.2.3.4"
                }
            }];
            return payload;
        },
        
        setImageManagerURL : function() {
            var url = App.info.actionURL + "imageManagerUpload";
            url += "?objectId=" + (App.page.getObjectId() || App.info.user.objectId);
            
            var $iframe = this.$element.find("iframe");
            $iframe.attr("src",url);
        },
        
        openSecurityContextSidePanel : function() {
            var options = {"options": 
                {"dashboardRef": "tvc:dashboard:helium:launchpad:common/SecurityContextSidepanel.xml"}
            };
            App.launchpad.openDashboardInSidePanel(options);
        },
        
        intro : function() {
            var intro = introJs();
            intro.setOptions({
              steps: [
                { 
                  intro: "Welcome to Helium!"
                },
                {
                  element: document.querySelector('#topbar'),
                  intro: "This topbar panel is always visible, and can serve as quick navigation."
                },
                {
                    element: document.querySelector('#topbarsearch'),
                    intro: 'You can search by entering a query and hitting enter or clicking the icon.',
                    position: 'bottom'
                  },                
                  {
                      element: document.querySelectorAll('i.icon.wifi')[1],
                      intro: 'Green = Online, Red = offline.',
                      position: 'bottom'
                    },                  
                {
                  element: document.querySelectorAll('#page-content')[0],
                  intro: "Actual page content goes here - configurable in Dashboard and Widgets"
                }
              ]
            });
            intro.start();            
        },
        
        formatDate : function (value) {
            console.log(value);
            return moment(value).format('MMM D');
        },
        
        reloadIssues : function() {
            var issuesWidget = App.page.getWidgetById("myissues");
            issuesWidget.refresh();
            App.toaster.success("Created issue");
        },
        
        previewFileInSidePanel : function(e, objectId, fileName) {
            var input = {
                "options": {
                    "dashboardRef": "tvc:dashboard:helium:launchpad:common/FilePreviewSidepanel.xml",
                    "row": {
                        "objectId": objectId,
                        "fileName": fileName
                    }
                }
            };
            App.launchpad.openDashboardInSidePanel(input);
            
            // Instead of postRender, as I need params.
            var boundFn = App.launchpad.filePreview.bind(input);
            setTimeout(boundFn, 1000);
        },
        
        filePreview : function() {
            var $iframe = App.page.sidepanel.$panel.find("iframe");
            
            // Append filename to header
            var $title = $iframe.find("span.title");
            $title.text($title.text() + " " + this.options.row.fileName);
            
            // Set iframe src attribute
            var url = App.info.contextURL + App.info.routingBase;
            url = url.replace("//goto","/goto");
            url += "/f/";
            url += App.page.getObjectId(); // this.options.row.objectId;
            url += "/generic/";
            url += this.options.row.fileName;

            $iframe.attr("src", url);
        },
        
        downloadFile : function() {
            console.log("dlfile");
        },
        
        toggleWidgetAtoB : function(fromId, toId) {
            var fromWidget = App.page.getWidgetById(fromId);
            var toWidget = App.page.getWidgetById(toId);
            if(fromWidget && toWidget) {
                App.Widget.toggleWidget(toWidget).bind(fromWidget);
            }
        },
        
        toggleStartPageWidget : function() {
            console.log("toggling..");
        },
        /*FUNCTIONS adopted from HEX: hex.js - END */
        
        /*FUNCTIONS adopted from HEX: searchFunctions.js - START */
        expandSections : function () {
            $('.he-search2.open .uip-search-form-section').find('.ti-f.ti-navright-f.expand-icon.collapsed').click();
        },

        collapseSections : function () {
            $('.he-search2.open .uip-search-form-section').find('.ti-f.ti-navright-f.expand-icon.expanded').click();
        },  
        /*FUNCTIONS adopted from HEX: searchFunctions.js - END */

        createConnectBookmarkDocument : function(input) {
            var data = this.selectedRows();
            var widgetName=this.widget.options.id;
            if (data.length === 0) {
                console.log("No rows Selected");
                App.log.error('Table: No row selected');
                return;
            }
            if (data[0].type.values[0].value === "Workspace Vault") {
                var objectIds = [];
                for ( var i = 0; i < data.length; i++) {
                    console.log(" rows Selected:"+data[i].objectId);
                    objectIds.push(data[i].objectId);
                }
            App.page.context.objectIds=objectIds;
            input.options = {formConfigName: 'tvc:form:helium:launchpad:common/CreateBookmarkDocument.xml', 
                fullscreen: true,
                connectWith : objectIds,
                formMode : 'CREATE_AND_CONNECT',
                updateServiceMethodMapping : {
                    'CREATE' : {
                        onSuccess : function() {
                            this.modal.close();
                            App.page.getWidgetById(widgetName).refresh();
                        }
                    }
                },
            };
            input.postRender = 'App.launchpad.createDocumentPostRender'
            App.form.createAndConnect(input);
            }
            else{
                console.log("Please Select Folder");
                App.log.error('Please Select Folder');
                return;
            }
        },

        createConnectIssue : function(input) {
            var widgetName=this.widget.options.id;
            var sourceObjectID = this.context.objectIds[0];
            if (sourceObjectID==""){
                        App.log.error('No source object');}
            var objectIds = [];
            objectIds.push(sourceObjectID);
            App.page.context.objectIds=objectIds;
            input.options.connectWith=objectIds;
            input.options.formMode='CREATE_AND_CONNECT';
            input.options.parentObjectId=sourceObjectID;
            input.options.updateServiceMethodMapping={
                'CREATE' : {
                    onSuccess : function() {
                        this.modal.close();
                        App.toaster.info('Issue was successfully created');
                        App.page.getWidgetById(widgetName).refresh();
                    }
                }
            };
            App.form.createAndConnect(input);
        },

        createDocument: function(input) {
            var widgetName=this.widget.options.id;
            var options = {
                options: {
                    formConfigName: input.options.formConfigName,
                    fullscreen: true,
                    formMode: 'CREATE_AND_CONNECT',
                    fullscreen: true,
                    modal : {
                        position : {
                            top:"20%",
                            left:"20%",
                            right:"20%",
                            bottom:"20%"
                        },
                        resizable:true,
                        draggable:true
                    },
                    updateServiceMethodMapping : {
                        'CREATE' : {
                            onSuccess : function() {
                                this.modal.close();
                                App.toaster.info('Object was successfully created');
                                App.page.getWidgetById(widgetName).refresh();
                            }
                        }
                    }
                },
                postRender : 'App.launchpad.createDocumentPostRender'
                };
            return App.form.createAndConnect(options);
        },

      createDocumentPostRender: function() {
            var form = this;
            var policyField = form.getFieldInstanceByDataIdentifier("policy");
            var typeField = form.getFieldInstanceByDataIdentifier("typeField");
            typeField.on('change', App.launchpad.onRequestTypeChange.bind(typeField, form));
            var value = [policyField.getValue()];
            var displayValue = [policyField.getValue()];
            policyField.schema.enum = value;
            policyField.options.optionLabels = displayValue;
            policyField.refresh();
            },

     onRequestTypeChange: function(form, typeField){
            var requestType = form.getFieldInstanceByDataIdentifier("typeField");
            var typeName=requestType.getValue();
            var args = JSON.stringify([typeName]);
            var policyField = form.getFieldInstanceByDataIdentifier("policy");
            App.ajax.call({
                url: App.ajax.getUrl("getPolicyForType", "getPolicy"),
                dataType: 'json',
                type: 'post',
                async: true,
                data: args
            }).then(function(data) {
                policyField.schema.enum = data.value;
                policyField.options.optionLabels = data.displayValue;
                policyField.data[0].text=data.value[0];
                policyField.data[0].value=data.value[0];
                policyField.refresh();
                console.log(data.displayValue);
                }).catch(function (error) {
                console.log(error);
                });
        }
    };

    App.launchpad = $.extend({}, App.launchpad || {}, launchpad);
})(window, App, jQuery);
